@@ -2,12 +2,10 @@
 import pytest
 
 from bowtie import App
-from bowtie.control import Button
 
 
 def test_subscribe_error():
     """Subscribe with incorrect argument order."""
     app = App()
-    button = Button()
-    with pytest.raises(TypeError):
-        app.subscribe(3, button.on_click)
+    with pytest.raises(IndexError):
+        app.subscribe()
